import json

none = "d3043820717d74d9a17694c176d39733"

# region GKE
class GKE:
	"""
	# Arguments
	name: str
	preemptible_percentage: int
	capacity: Capacity
	instance_types: InstanceTypes
	availability_zones: list[str]
	node_image: str
	"""
	def __init__(
			self,
			name=none,
			preemptible_percentage=none,
			capacity=none,
			instance_types=none,
			availability_zones=none,
			node_image=none):

		self.name = name
		self.preemptible_percentage = preemptible_percentage
		self.capacity = capacity
		self.instance_types = instance_types
		self.availability_zones = availability_zones
		self.node_image = node_image

class Capacity:
	"""
	# Arguments
	minimum: int
	maximum: int
	target: int
	"""
	def __init__(
			self,
			minimum=none,
			maximum=none,
			target=none):

		self.minimum = minimum
		self.maximum = maximum
		self.target = target

class InstanceTypes:
	"""
	# Arguments
	ondemand: str
	preemptible: list[str]
	"""
	def __init__(self, ondemand=none, preemptible=none):

		self.ondemand = ondemand
		self.preemptible = preemptible

class ImportGKERequest:
    def __init__(self, group):
        self.group = group

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# endregion


import json

none = "d3043820717d74d9a17694c176d39733"

# region GcpCredentials
class GcpCredentials:
    """
    # Arguments
    serviceAccount: ServiceAccount
    """
    def __init__(
            self, 
            serviceAccount):

        self.serviceAccount = serviceAccount

class ServiceAccount:
    """
    # Arguments
    type: str
    project_id: str
    private_key_id: str
    private_key: str
    client_email: str
    client_id: str
    auth_uri: str
    token_uri: str
    auth_provider_x509_cert_url: str
    client_x509_cert_url: str
    """
    def __init__(
            self, 
            type,
            project_id,
            private_key_id,
            private_key,
            client_email,
            client_id,
            auth_uri,
            token_uri,
            auth_provider_x509_cert_url,
            client_x509_cert_url):

        self.type = type
        self.project_id = project_id
        self.private_key_id = private_key_id
        self.private_key = private_key
        self.client_email = client_email
        self.client_id = client_id
        self.auth_uri = auth_uri
        self.token_uri = token_uri
        self.auth_provider_x509_cert_url = auth_provider_x509_cert_url
        self.client_x509_cert_url = client_x509_cert_url

class GcpSetCredentialsRequest:
    """
    # Arguments
    gcp_credentials: GcpCredentials
    """
    def __init__(self, gcp_credentials):
        self.serviceAccount = gcp_credentials.serviceAccount

    def to_json(self):
        return json.dumps(
            self,
            default=lambda o: o.__dict__,
            sort_keys=True,
            indent=4)
# endregion


import json

none = "d3043820717d74d9a17694c176d39733"

# region EMR
class EMR:
	"""
	# Arguments
	name: str
	decription: str
	region: str
	strategy: Strategy
	compute: Compute
	cluster: Cluster
	scheduling: Scheduling
	scaling: Scaling
	"""
	def __init__(
		self,
		name=none,
		description=none,
		region=none,
		strategy=none,
		compute=none,
		cluster=none,
		scheduling=none,
		scaling=none):

		self.name = name
		self.description = description
		self.region = region
		self.strategy = strategy
		self.compute = compute
		self.cluster = cluster
		self.scheduling = scheduling
		self.scaling = scaling

# endregion

# region Strategy
class Strategy:
	"""
	# Arguments
	wrapping: Wrapping
	cloning: Cloning
	new: Newing
	provisioning_timeout: ProvisioningTimeout
	"""
	def __init__(
		self,
		wrapping=none,
		cloning=none,
		new=none,
		provisioning_timeout=none):

		self.wrapping = wrapping
		self.cloning = cloning
		self.new = new
		self.provisioning_timeout = provisioning_timeout

class Wrapping:
	"""
	# Arguments
	source_cluster_id: str
	"""
	def __init__(
		self,
		source_cluster_id):

		self.source_cluster_id = source_cluster_id

class Cloning:
	"""
	# Arguments
	origin_cluster_id: str
	include_steps: bool
	number_of_retries: int
	"""
	def __init__(
		self,
		origin_cluster_id=none,
		include_steps=none,
		number_of_retries=none):

		self.origin_cluster_id = origin_cluster_id
		self.include_steps = include_steps
		self.number_of_retries=number_of_retries

class New:
	"""
	# Arguments
	release_label: str
	number_of_retries: int
	"""
	def __init__(
		self,
		release_label=none,
		number_of_retries=none):

		self.release_label=release_label
		self.number_of_retries=number_of_retries

class ProvisioningTimeout:
	"""
	# Arguments
	timeout: int
	timeout_action: str
	"""
	def __init__(
		self, 
		timeout = none,
		timeout_action = none):

		self.timeout = timeout
		self.timeout_action = timeout_action


# endregion

# region Compute
class Compute:
	"""
	# Arguments
	ebs_root_volume_size: int
	availability_zones: List[AvailabilityZone]
	bootstrap_actions: BootstrapActions
	steps: Steps
	instance_groups: InstanceGroups
	configurations: Configurations
	emr_managed_master_security_group: str
	emr_managed_slave_security_group: str
	additional_master_security_groups: List[str]
	service_access_security_group: str
	custom_ami_id: str
	repo_upgrade_on_boot: str
	additional_slave_security_groups: List[str]
	ec2_key_name: str
	applications: List[Application]
	"""
	def __init__(
		self,
		ebs_root_volume_size=none,
		availability_zones=none,
		bootstrap_actions=none,
		steps=none,
		instance_groups=none,
		configurations=none,
		emr_managed_master_security_group=none,
		emr_managed_slave_security_group=none,
		additional_master_security_groups=none,
		service_access_security_group=none,
		custom_ami_id=none,
		repo_upgrade_on_boot=none,
		additional_slave_security_groups=none,
		ec2_key_name=none,
		applications=none):

		self.ebs_root_volume_size = ebs_root_volume_size
		self.availability_zones = availability_zones
		self.bootstrap_actions = bootstrap_actions
		self.steps = steps
		self.instance_groups = instance_groups
		self.configurations = configurations
		self.emr_managed_master_security_group = emr_managed_master_security_group
		self.emr_managed_slave_security_group = emr_managed_slave_security_group
		self.additional_master_security_groups = additional_master_security_groups
		self.service_access_security_group = service_access_security_group
		self.custom_ami_id = custom_ami_id
		self.repo_upgrade_on_boot = repo_upgrade_on_boot
		self.additional_slave_security_groups = additional_slave_security_groups
		self.ec2_key_name = ec2_key_name
		self.applications = applications

class AvailabilityZone:
	"""
	# Arguments
	name: str
	subnetId: str
	"""
	def __init__(
		self,
		name=none,
		subnetId=none):

		self.name = name
		self.subnetId = subnetId

class BootstrapActions:
	"""
	# Arguments
	file: File
	"""
	def __init__(
		self,
		file=none):

		self.file = file

class File:
	"""
	# Arguments
	bucket: str
	key: str
	"""
	def __init__(
		self, 
		bucket=none,
		key=none):

		self.bucket = bucket
		self. key = key

class Steps:
	"""
	# Arguments
	file: File
	"""
	def __init__(
		self,
		file=none):

		self.file = file

class Application:
	"""
	# Arguments
	name: str
	args: List[str]
	version: str
	"""
	def __init__(
		self,
		name=none,
		args=none,
		version=none):
	
		self.name = name
		self.args = args
		self.version = version

class InstanceGroups:
	"""
	# Arguments
	master_group: MasterGroup
	core_group: CoreGroup
	task_group: TaskGroup
	"""
	def __init__(
		self, 
		master_group=none,
		core_group=none,
		task_group=none):

		self.master_group = master_group
		self.core_group = core_group
		self.task_group = task_group

class MasterGroup:
	"""
	# Arguments
	instance_types: List[str]
	target: int
	life_cycle: str
	configurations: Configurations
	"""
	def __init__(
		self,
		instance_types=none,
		target=none,
		life_cycle=none,
		configurations=none):

		self.instance_types = instance_types
		self.target = target
		self.life_cycle = life_cycle
		self.configurations = configurations

class CoreGroup:
	"""
	# Arguments
	instance_types: List[str]
	target: int
	capacity: Capacity
	life_cycle: str
	ebs_configuration: EbsConfiguration
	configurations: Configurations
	"""
	def __init__(
		self,
		instance_types=none,
		target=none,
		capacity=none,
		life_cycle=none,
		ebs_configuration=none,
		configurations=none):

		self.instance_types = instance_types
		self.target = target
		self.capacity = capacity
		self.life_cycle = life_cycle
		self.ebs_configuration = ebs_configuration
		self.configurations = configurations

class TaskGroup:
	"""
	# Arguments
	instance_types: List[str]
	capacity: Capacity
	life_cycle: str
	ebs_configuration: EbsConfiguration
	configurations: Configurations
	"""
	def __init__(
		self,
		instance_types=none,
		capacity=none,
		life_cycle=none,
		ebs_configuration=none,
		configurations=none):

		self.instance_types = instance_types
		self.capacity = capacity
		self.life_cycle = life_cycle
		self.ebs_configuration = ebs_configuration
		self.configurations = configurations

class Capacity:
	"""
	# Arguments
	target: int
	minimum: int
	maximum: int
	unit: str
	"""
	def __init__(
		self,
		target=none,
		minimum=none,
		maximum=none,
		unit=none):

		self.target = target
		self.minimum = minimum
		self.maximum = maximum
		self.unit = unit

class EbsConfiguration:
	"""
	# Arguments
	ebs_block_device_configs: List[SingleEbsConfig]
	ebs_optimized: bool
	"""
	def __init__(
		self,
		ebs_block_device_configs=none,
		ebs_optimized=none):

		self.ebs_block_device_configs = ebs_block_device_configs
		self.ebs_optimized = ebs_optimized

class SingleEbsConfig:
	"""
	# Arguments
	volume_specification: VolumeSpecification
	volumes_per_instance: int
	"""
	def __init__(
		self,
		volume_specification=none,
		volumes_per_instance=none):

		self.volume_specification = volume_specification
		self.volumes_per_instance = volumes_per_instance

class VolumeSpecification:
	"""
	# Arguments
	volume_type: str
	size_in_GB: int
	"""
	def __init__(
		self,
		volume_type=none,
		size_in_gb=none):

		self.volume_type = volume_type
		self.size_in_gB = size_in_gb

class Configurations:
	"""
	# Arguments
	file: File
	"""
	def __init__(
		self,
		file=none):

		self.file = file
# endregion

# region Cluster
class Cluster:
	"""
	# Arguments
	visible_to_all_users: Boolean
	termination_protected: Boolean
	keep_job_flow_alive_when_no_steps: Boolean
	log_uri: str
	additional_info: str
	job_flow_role: str
	security_configuration: str
	"""
	def __init__(
		self,
		visible_to_all_users=none,
		termination_protected=none,
		keep_job_flow_alive_when_no_steps=none,
		log_uri=none,
		additional_info=none,
		job_flow_role=none,
		security_configuration=none):

		self.visible_to_all_users = visible_to_all_users
		self.termination_protected = termination_protected
		self.keep_job_flow_alive_when_no_steps = keep_job_flow_alive_when_no_steps
		self.log_uri = log_uri
		self.additional_info = additional_info
		self.job_flow_role = job_flow_role
		self.security_configuration = security_configuration
# endregion

# region Scheduling
class Scheduling:
	"""
	# Arguments 
	tasks: List[Task]
	"""
	def __init__(
		self,
		tasks=none):

		self.tasks = tasks

class Task:
	"""
	# Arguments
	is_enabled: bool
	instance_group_type: str
	task_type: str
	cron_expression: str
	target_capacity: int
	min_capacity: int
	max_capacity: int
	"""
	def __init__(
		self,
		is_enabled=none,
		instance_group_type=none,
		task_type=none,
		cron_expression=none,
		target_capacity=none,
		min_capacity=none,
		max_capacity=none):

		self.is_enabled = is_enabled
		self.instance_group_type = instance_group_type
		self.task_type = task_type
		self.cron_expression = cron_expression
		self.target_capacity = target_capacity
		self.min_capacity = min_capacity
		self.max_capacity = max_capacity

# endregion



# region Scaling
class Scaling:
	"""
	# Arguments
	up: List[Metric]
	down: List[Metric]
	"""
	def __init__(
		self,
		up=none,
		down=none):

		self.up = up
		self.down = down


class Metric:
	"""
	# Arguments
	metric_name: str
	statistic: str
	unit: str
	threshold: int
	adjustment: int
	namespace: str
	period: int
	evaluation_periods: int
	action: Action
	cooldown: int
	dimensions: List[Dimension]
	operator: str
	"""
	def __init__(
		self,
		metric_name=none,
		statistic=none,
		unit=none,
		threshold=none,
		adjustment=none,
		namespace=none,
		period=none,
		evaluation_periods=none,
		action=none,
		cooldown=none,
		dimensions=none,
		operator=none):

		self.metric_name = metric_name
		self.statistic = statistic
		self.unit = unit
		self.threshold = threshold
		self.adjustment = adjustment
		self.namespace = namespace
		self.period = period
		self.evaluation_periods = evaluation_periods
		self.action = action
		self.cooldown = cooldown
		self.dimensions = dimensions
		self.operator = operator

class Action:
	"""
	# Arguments
	type: str
	adjustment: int
	min_target_capacity: int
	target: int
	minimum: int
	maximum: int
	"""
	def __init__(
		self,
		type=none,
		adjustment=none,
		min_target_capacity=none,
		target=none,
		minimum=none,
		maximum=none):

		self.type = type
		self.adjustment = adjustment
		self.min_target_capacity = min_target_capacity
		self.target = target
		self.minimum = minimum
		self.maximum = maximum

class Dimension:
	"""
	# Arguments
	name: str
	"""
	def __init__(
		self,
		name=none):

		self.name = name


#endregion



class EMRCreationRequest:
    def __init__(self, mrScaler):
        self.mrScaler = mrScaler

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)










import json
from typing import List

none = "d3043820717d74d9a17694c176d39733"


# region Canary Scale
class BackgroundVerification:
    """
    # Arguments
    template_names: List[str]
    """

    def __init__(
            self,
            template_names: List[str] = none):
        self.template_names = template_names


class SetCanaryScale:
    """
    # Arguments
    match_traffic_weight: bool
    replicas: int
    weight: int
    """

    def __init__(
            self,
            match_traffic_weight: bool = none,
            replicas: int = none,
            weight: int = none):
        self.match_traffic_weight = match_traffic_weight
        self.replicas = replicas
        self.weight = weight


class Verification:
    """
    # Arguments
    template_names: List[str]
    """

    def __init__(
            self,
            template_names: List[str] = none):
        self.template_names = template_names


class Pause:
    """
    # Arguments
    duration: str
    """

    def __init__(
            self,
            duration: str = none):
        self.duration = duration


class HeaderValue:
    """
    # Arguments
    exact: str
    prefix: str
    regex: str
    """

    def __init__(
            self,
            exact: str = none,
            prefix: str = none,
            regex: str = none):
        self.exact = exact
        self.prefix = prefix
        self.regex = regex


class Match:
    """
    # Arguments
    header_name: str
    header_value: HeaderValue
    """

    def __init__(
            self,
            header_name: str = none,
            header_value: HeaderValue = none):
        self.header_name = header_name
        self.header_value = header_value


class HeaderRoute:
    """
    # Arguments
    name: str
    match: List[Match]
    """

    def __init__(
            self,
            name: str = none,
            match: List[Match] = none):
        self.name = name
        self.match = match


class CanarySteps:
    """
    # Arguments
    name: str
    set_weight: int
    set_canary_scale: SetCanaryScale
    verification: Verification
    pause: Pause
    set_header_route: HeaderRoute
    """

    def __init__(
            self,
            name: str = none,
            set_weight: int = none,
            set_canary_scale: SetCanaryScale = none,
            verification: Verification = none,
            pause: Pause = none,
            set_header_route: HeaderRoute = none):
        self.name = name
        self.set_weight = set_weight
        self.set_canary_scale = set_canary_scale
        self.verification = verification
        self.pause = pause
        self.set_header_route = set_header_route


class Canary:
    """
    # Arguments
    background_verification: BackgroundVerification
    steps: List[CanarySteps]
    """

    def __init__(
            self,
            background_verification: BackgroundVerification = none,
            steps: List[CanarySteps] = none):
        self.background_verification = background_verification
        self.steps = steps


# endregion


# region Rolling
class RollingSteps:
    """
    # Arguments
    name: str
    pause: Pause
    verification: Verification
    """

    def __init__(
            self,
            name: str = none,
            pause: Pause = none,
            verification: Verification = none):
        self.name = name
        self.pause = pause
        self.verification = verification


# endregion


# region
class Strategy:
    """
    # Arguments
    name: str
    canary: Canary
    rolling: RollingSteps
    """

    def __init__(
            self,
            name: str = none,
            canary: Canary = none,
            rolling: RollingSteps = none):
        self.name = name
        self.canary = canary
        self.rolling = rolling


# endregion


# region Client Requests
class CreateStrategyRequest:
    """
    # Arguments
    strategy : Strategy
    """

    def __init__(self, strategy: Strategy):
        self.strategy = strategy

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion



import json

none = "d3043820717d74d9a17694c176d39733"

# region Balancer
class Balancer:
	"""
	# Arguments
	name: str
	timeouts: Timeout
	tags: List[Tag]
	"""
	def __init__(
		self,
		name=none,
		timeouts=none,
		tags=none):

		self.name = name
		self.timeouts = timeouts
		self.tags = tags

class Timeout:
	"""
	# Arguments
	draining: int
	idle: int
	"""
	def __init__(
		self,
		draining=none,
		idle=none):

		self.draining = draining
		self.idle = idle

class Tag:
	"""
	# Arguments
	key: str
	value: str
	"""
	def __init__(
		self,
		key=none,
		value=none):

		self.key = key
		self.value = value

class BalancerRequest:
    def __init__(self, balancer):
        self.balancer = balancer

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# endregion



# region Target Set

class TargetSet:
	"""
	# Arguments
	name: str
	balancer_id: str
	deployment_id: str
	protocol: str
	weight: int
	config: Config
	integrations: Integrations
	health_check: HealthCheck
	tags: List[Tag]
	"""
	def __init__(
		self,
		name=none,
		balancer_id=none,
		deployment_id=none,
		protocol=none,
		weight=none,
		config=none,
		integrations=none,
		health_check=none,
		tags=none):

		self.name=name
		self.balancer_id=balancer_id
		self.deployment_id=deployment_id
		self.protocol=protocol
		self.weight=weight
		self.config=config
		self.integrations=integrations
		self.health_check=health_check
		self.tags=tags

class Config:
	"""
	# Argument
	rate_limiter: RateLimiter
	"""
	def __init__(
		self,
		rate_limiter=none):

		self.rate_limiter=rate_limiter

class RateLimiter:
	"""
	# Argument
	request_per_second: int
	"""
	def __init__(
		self,
		requests_per_second=none):

		self.requests_per_second=requests_per_second

class Integrations:
	"""
	# Arguments
	ecs: List[ECS]
	"""
	def __init__(
		self,
		ecs=none):

		self.ecs=ecs

class ECS:
	"""
	# Arguments
	target_group_arn: str
	target_group_name: str
	region: str
	service_name: str
	cluster_name: str
	"""
	def __init__(
		self,
		target_group_arn=none,
		target_group_name=none,
		region=none,
		service_name=none,
		cluster_name=none):

		self.target_group_arn=target_group_arn
		self.target_group_name=target_group_name
		self.region=region
		self.service_name=service_name
		self.cluster_name=cluster_name

class HealthCheck:
	"""
	# Arguments
	interval: int
	path: str
	port: int
	protocol: str
	timeout: int
	healthy_threshold_count: int
	unhealthy_threshold_count: int
	"""
	def __init__(
		self,
		interval=none,
		path=none,
		port=none,
		protocol=none,
		timeout=none,
		healthy_threshold_count=none,
		unhealthy_threshold_count=none):

		self.interval = interval
		self.path = path
		self.port = port
		self.protocol = protocol
		self.timeout = timeout
		self.healthy_threshold_count = healthy_threshold_count
		self.unhealthy_threshold_count = unhealthy_threshold_count

class TargetSetRequest:
    def __init__(self, target_set):
        self.target_set = target_set

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# endregion




# region Target

class Target:
	"""
	# Arguments
	id: str (Deregister Target)
	balancer_id: (Create Target)
	target_set_id: (Create Target)
	name: str (Create & Register Target)
	host: str (Create & Register Target)
	port: int (Create Target)
	weight: int (Create & Register Target)
	tags: List[Tags] (Create & Register Target)
	"""
	def __init__(
		self,
		tags=none,
		name=none,
		id=none,
		balancer_id=none,
		port=none,
		target_set_id=none,
		host=none,
		weight=none):

		self.id=id
		self.balancer_id=balancer_id
		self.target_set_id=target_set_id
		self.name=name
		self.port=port
		self.host=host
		self.weight=weight
		self.tags=tags


class TargetsRequest:
    def __init__(self, targets):
        self.targets = targets

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

class TargetRequest:
    def __init__(self, target):
        self.target = target

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# endregion




# region Middleware

class Middleware:
	"""
	# Arguments
	balancer_id: str
	type: str
	priority: int
	spec: Spec
	tags: List[Tag]
	"""
	def __init__(
		self,
		balancer_id=none,
		type=none,
		priority=none,
		spec=none,
		tags=none):

		self.balancer_id=balancer_id
		self.type=type
		self.priority=priority
		self.spec=spec
		self.tags=tags

class Spec:
	"""
	# Arguments
	action: str
	conditions: List[Condition]
	"""
	def __init__(
		self,
		action=none,
		conditions=none):

		self.action=action
		self.conditions=conditions


class Condition:
	"""
	# Arguments
	type: str
	values: List[str]
	"""
	def __init__(
		self,
		type=none,
		values=none):

		self.type=type
		self.values=values

class MiddlewareRequest:
    def __init__(self, middleware):
        self.middleware = middleware

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# endregion




# region Listener

class Listener:
	"""
	# Arguments
	balancer_id: str
	protocol: str
	port: int
	tls_config: TLSConfig
	tags: List[Tag]
	"""
	def __init__(
		self,
		balancer_id=none,
		protocol=none,
		port=none,
		tls_config=none,
		tags=none):

		self.balancer_id=balancer_id
		self.protocol=protocol
		self.port=port
		self.tls_config=tls_config
		self.tags=tags

class TLSConfig:
	"""
	# Arguments
	min_version: str
	max_version: str
	session_ticket_disabled: bool
	prefer_service_cipher_suites: bool
	cipher_suites: List[str]
	insecure_skip_verify: bool
	certificate_ids: List[str]
	"""
	def __init__(
		self,
		min_version=none,
		max_version=none,
		session_tickets_disabled=none,
		prefer_server_cipher_suites=none,
		cipher_suites=none,
		insecure_skip_verify=none,
		certificate_ids=none):

		self.min_version=min_version
		self.max_version=max_version
		self.session_tickets_disabled=session_tickets_disabled
		self.prefer_server_cipher_suites=prefer_server_cipher_suites
		self.cipher_suites=cipher_suites
		self.insecure_skip_verify=insecure_skip_verify
		self.certificate_ids=certificate_ids


class ListenerRequest:
    def __init__(self, listener):
        self.listener = listener

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)

# endregion




# region Routing Rule

class RoutingRule:
	"""
	# Arguments
	balancer_id: str
	route: str
	target_set_ids: List[str]
	middleware_ids: List[str]
	listener_id: str
	priority: int
	tags: List[Tag]
	"""
	def __init__(
		self,
		balancer_id=none,
		route=none,
		target_set_ids=none,
		middleware_ids=none,
		listener_id=none,
		priority=none,
		tags=none):

		self.balancer_id=balancer_id
		self.route=route
		self.target_set_ids=target_set_ids
		self.middleware_ids=middleware_ids
		self.listener_id=listener_id
		self.priority=priority
		self.tags=tags

class RoutingRuleRequest:
    def __init__(self, routing_rule):
        self.routing_rule = routing_rule

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)








import json

from spotinst_sdk2.client import Client
import spotinst_sdk2.models.ocean.aws as aws_ocean


class OceanAwsClient(Client):
    __base_ocean_cluster_url = "/ocean/aws/k8s/cluster"
    __base_ocean_launchspec_url = "/ocean/aws/k8s/launchSpec"
    __base_ocean_url = "/ocean/k8s/cluster/"

    def create_ocean_cluster(self, ocean: aws_ocean.Ocean):
        """
        Create an Ocean Cluster 
        
        # Arguments
        ocean (Ocean): Ocean Object

        # Returns
        (Object): Ocean API response 
        """
        ocean = aws_ocean.OceanRequest(ocean)

        excluded_missing_dict = self.exclude_missing(json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url,
            entity_name='ocean')

        formatted_response = self.convert_json(response, 
                                               self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_ocean_cluster(self, ocean_id: str, ocean: aws_ocean.Ocean, auto_apply_tags: str = None):
        """
        Update an existing Ocean Cluster 
        
        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        auto_apply_tags (String): Option to update instance tags on the fly without rolling the cluster.
        ocean (Ocean): Ocean object
        
        # Returns
        (Object): Ocean API response 
        """
        ocean = aws_ocean.OceanRequest(ocean)

        excluded_missing_dict = self.exclude_missing(json.loads(ocean.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put_with_params(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name='ocean',
            user_query_params=dict(autoApplyTags=auto_apply_tags))

        formatted_response = self.convert_json(
            response,
            self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_ocean_cluster(self):
        """
        List the configurations for all Ocean clusters in the specified account.
        
        # Returns
        (Object): Ocean API response 
        """

        response = self.send_get(
            url=self.__base_ocean_cluster_url,
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_all_ocean_sizing(self, ocean_id: str, namespace: str):
        """
        Get all right sizing recommendations for an Ocean cluster

        # Returns
        (Object): Ocean API response
        """
        response = self.send_get(
            url=(self.__base_ocean_cluster_url +
                 "/" + ocean_id +
                 "/rightSizing/suggestion?namespace={}"
                 ).format(namespace),
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def fetch_rightsizing_recommendations(self, ocean_id: str, filter: aws_ocean.RightSizingRecommendationFilter = None):
        """
        Get right-sizing recommendations for an Ocean cluster and filter them according to namespace or label.

        # Arguments
        ocean_id (String): Id of the Ocean Cluster
        filter (RightSizingRecommendationFilter): Optional - may be null.

        # Returns
        (Object): Ocean API response
        """
        recommendation_request = aws_ocean.RightSizingRecommendationRequest(filter)

        excluded_missing_dict = self.exclude_missing(json.loads(recommendation_request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        group_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url +
            "/" + ocean_id + "/rightSizing/suggestion",
            entity_name='ocean')

        formatted_response = self.convert_json(
            group_response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_ocean_cluster(self, ocean_id: str):
        """
        Get an exsisting Ocean Cluster json

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean API response 
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_ocean_cluster(self, ocean_id: str):
        """
        Delete an Ocean Cluster

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean API response
        """
        return self.send_delete(
            url=self.__base_ocean_cluster_url + "/" + ocean_id,
            entity_name="ocean"
        )

    def get_aggregated_cluster_costs(self, ocean_id: str, aggregated_cluster_costs: aws_ocean.AggregatedClusterCosts):
        """
        Get aggregated cluster costs

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        aggregated_cluster_costs (AggregatedClusterCosts): Aggregated Cluster Costs request

        # Returns
        (Object): Aggregated Cluster Costs API response
        """
        aggregated_cluster_costs_request = aws_ocean.AggregatedClusterCostRequest(aggregated_cluster_costs)

        excluded_missing_dict = self.exclude_missing(json.loads(aggregated_cluster_costs_request.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        aggregated_costs_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/aggregatedCosts",
            entity_name='ocean (aggregated cluster costs)')

        formatted_response = self.convert_json(
            aggregated_costs_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def initiate_roll(self, ocean_id: str, cluster_roll: aws_ocean.Roll):
        """
        Initiate Cluster Rolls

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        cluster_roll (Roll): Cluster Roll / Roll with Instance Ids/ Launch specification Ids

        # Returns
        (Object): Cluster Roll API response
        """
        roll_request = aws_ocean.ClusterRollInitiateRequest(cluster_roll)

        excluded_missing_dict = self.exclude_missing(json.loads(roll_request.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        aggregated_costs_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll",
            entity_name='ocean (Cluster Roll)')

        formatted_response = self.convert_json(
            aggregated_costs_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def list_rolls(self, ocean_id: str):
        """
        Get status for all rolls of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): List of Cluster Roll API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll",
            entity_name="ocean (Cluster Roll)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def update_roll(self, ocean_id: str, roll_id: str, status: str):
        """
        Update a roll of an Ocean cluster.
        Performing the request will stop the next batch in a roll.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        roll_id (String): Ocean cluster roll identifier
        update_roll (UpdateRoll): update roll request

        # Returns
        (Object): Cluster Roll API response
        """
        update_roll_request = aws_ocean.ClusterRollUpdateRequest(status)

        excluded_missing_dict = self.exclude_missing(json.loads(update_roll_request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll/" + roll_id,
            entity_name='ocean (Cluster Roll)')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_roll(self, ocean_id: str, roll_id: str):
        """
        Get status for a roll of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        account_id (String): The ID of the account associated with your token.
        roll_id (String): Ocean cluster roll identifier

        # Returns
        (Object): Cluster Roll API response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/roll/" + roll_id,
            entity_name="ocean (Cluster Roll)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_cluster_nodes(self, ocean_id: str, instance_id: str = None, launch_spec_id: str = None):
        """
        Get nodes data of an Ocean cluster.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        instance_id (String): Get a specific node by instance id
        launch_spec_id (String): Ocean cluster launch specification identifier.

        # Returns
        (Object): Ocean Kubernetes AWS Nodes Data response
        """
        query_params = dict(instanceId = instance_id, launchSpecId = launch_spec_id)
        
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/nodes",
            entity_name="ocean (Cluster Nodes)",
            query_params=query_params
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_heartbeat_status(self, ocean_id: str):
        """
        Get the heartbeat status of the Ocean Controller for the cluster.
        The response returns the heartbeat status and the last heartbeat timestamp.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean Get Heartbeat response
        """
        response = self.send_get(
            url=self.__base_ocean_url + ocean_id + "/controllerHeartbeat",
            entity_name="ocean (Cluster Heartbeat)"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def instance_types_filter_simulation(self, ocean_id: str,
                                         instance_type_filter: aws_ocean.InstanceTypesFilters):
        """
        Returns all instances types that match the given filters.
        These instance types will be used if the cluster is configured with these filters.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster
        instance_type_filter (InstanceTypesFilters): The Instance types that match with all filters compose the
        Ocean's whitelist parameter. Cannot be configured together with whitelist/blacklist.

        # Returns
        (Object): Ocean Instance Types Filter Simulation response
        """
        group_dict = aws_ocean.InstanceTypesFilterSimulationRequest(instance_type_filter)

        excluded_missing_dict = self.exclude_missing(json.loads(group_dict.toJSON()))

        formatted_missing_dict = self.convert_json_with_list_of_lists(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        aggregated_costs_response = self.send_post(
            body=body_json,
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/instanceTypeFiltersSimulation",
            entity_name='ocean')

        formatted_response = self.convert_json(
            aggregated_costs_response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def allowed_instance_types(self, ocean_id: str):
        """
        Return the list of the allowed Ocean cluster instance types.

        # Arguments
        ocean_id (String): ID of the Ocean Cluster

        # Returns
        (Object): Ocean Allowed Instance Types response
        """
        response = self.send_get(
            url=self.__base_ocean_cluster_url + "/" + ocean_id + "/allowedInstanceTypes",
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_virtual_node_group(self, ocean_launch_spec_id: str):
        """
        Get Virtual Node Group of the cluster

        # Arguments
        ocean_launch_spec_id (String): Ocean cluster launch specification identifier

        # Returns
        (Object): Ocean Allowed Instance Types response
        """
        response = self.send_get(
            url=self.__base_ocean_launchspec_url + "/" + ocean_launch_spec_id,
            entity_name="ocean"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def launch_nodes_in_vng(self, ocean_launch_spec_id: str, amount: int):
        """
        Launch nodes in Virtual Node Group.

        # Arguments
        ocean_launch_spec_id (String): Ocean cluster launch specification identifier.
        launch_nodes (LaunchNodes): Object specifying the details for the launch request.

        # Returns
        (Object): Ocean Virtual Node Group Launch API response
        """
        launch_node_request = aws_ocean.LaunchNodesRequest(amount)

        excluded_missing_dict = self.exclude_missing(json.loads(launch_node_request.toJSON()))

        formatted_missing_dict = self.convert_json(
            excluded_missing_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_missing_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_ocean_launchspec_url + "/" + ocean_launch_spec_id + "/launchNodes",
            entity_name='ocean (Cluster Roll)')

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]



import json
from typing import List

none = "d3043820717d74d9a17694c176d39733"


class Persistence:
    """
    # Arguments
    persist_root_device: bool
    persist_block_devices: bool
    block_devices_mode: str
    persist_private_ip: bool
    """
    def __init__(self, persist_root_device: bool = none, persist_block_devices: bool = none,
                 block_devices_mode: str = none, persist_private_ip: bool = none):
        self.persist_root_device = persist_root_device
        self.persist_block_devices = persist_block_devices
        self.block_devices_mode = block_devices_mode
        self.persist_private_ip = persist_private_ip


class HealthCheck:
    """
    # Arguments
    type: bool
    auto_healing: bool
    grace_period: int
    unhealthy_duration: int
    """
    def __init__(self, type: str = none, auto_healing: bool = none, grace_period: int = none,
                 unhealthy_duration: int = none):
        self.type = type
        self.auto_healing = auto_healing
        self.grace_period = grace_period
        self.unhealthy_duration = unhealthy_duration


class Task:
    """
    # Arguments
    task_type: str
    start_time: str
    cron_expression: str
    is_enabled: bool
    frequency: str
    """
    def __init__(self, task_type: str = none, start_time: str = none, cron_expression: str = none,
                 is_enabled: bool = none, frequency: str = none):
        self.is_enabled = is_enabled
        self.frequency = frequency
        self.start_time = start_time
        self.cron_expression = cron_expression
        self.task_type = task_type


class Scheduling:
    """
    # Arguments
    tasks: list[Task]
    """
    def __init__(self, tasks: List[Task] = none):
        self.tasks = tasks


class RevertToSpot:
    """
    # Arguments
    perform_at: str
    """
    def __init__(self, perform_at: str = none):
        self.perform_at = perform_at


class Strategy:
    """
    # Arguments
    life_cycle: str
    orientation: str
    draining_timeout: int
    fallback_to_od: bool
    utilize_reserved_instances: bool
    utilize_commitments: bool
    optimization_windows: list[str]
    revert_to_spot: RevertToSpot
    minimum_instance_lifetime: int
    """
    def __init__(self, life_cycle: str = none, orientation: str = none, draining_timeout: int = none,
                 fallback_to_od: bool = none,
                 utilize_reserved_instances: bool = none, utilize_commitments: bool = none,
                 optimization_windows: List[str] = none,
                 revert_to_spot: RevertToSpot = none, minimum_instance_lifetime: int = none):
        self.life_cycle = life_cycle
        self.orientation = orientation
        self.draining_timeout = draining_timeout
        self.fallback_to_od = fallback_to_od
        self.utilize_reserved_instances = utilize_reserved_instances
        self.utilize_commitments = utilize_commitments
        self.optimization_windows = optimization_windows
        self.revert_to_spot = revert_to_spot
        self.minimum_instance_lifetime = minimum_instance_lifetime


class EBS:
    """
    # Arguments
    delete_on_termination: bool
    encrypted: bool
    iops: int
    throughput: float
    volume_size: int
    volume_type: str
    kms_key_id: str
    snapshot_id: str
    """
    def __init__(self, delete_on_termination: bool = none, encrypted: bool = none, iops: int = none,
                 throughput: float = none,
                 kms_key_id: str = none,
                 snapshot_id: str = none,
                 volume_size: int = none,
                 volume_type: str = none):
        self.delete_on_termination = delete_on_termination
        self.encrypted = encrypted
        self.iops = iops
        self.throughput = throughput
        self.volume_size = volume_size
        self.volume_type = volume_type
        self.kms_key_id = kms_key_id
        self.snapshot_id = snapshot_id


class BlockDeviceMapping:
    """
    # Arguments
    device_name: str
    ebs: EBS
    no_device: str
    virtual_name: str
    """
    def __init__(self, device_name: str = none, ebs: EBS = none, no_device: str = none, virtual_name: str = none):
        self.device_name = device_name
        self.ebs = ebs
        self.no_device = no_device
        self.virtual_name = virtual_name


class CreditSpecification:
    """
    # Arguments
    cpu_credits: str
    """
    def __init__(self, cpu_credits: str = none):
        self.cpu_credits = cpu_credits


class IamRole:
    """
    # Arguments
    name: str
    arn: str
    """
    def __init__(self, name: str = none, arn: str = none):
        self.name = name
        self.arn = arn


class InstanceTypes:
    """
    # Arguments
    preferred_type: str
    types: list[str]
    """
    def __init__(self, preferred_type: str = none, types: List[str] = none):
        self.preferred_type = preferred_type
        self.types = types


class NetworkInterface:
    """
    # Arguments
    device_index: int
    associate_public_ip_address: bool
    associate_ipv6_address: bool
    """
    def __init__(self, device_index: int = none, associate_public_ip_address: bool = none,
                 associate_ipv6_address: bool = none):
        self.device_index = device_index
        self.associate_public_ip_address = associate_public_ip_address
        self.associate_ipv6_address = associate_ipv6_address


class TagSpecification:
    """
    # Arguments
    should_tag: bool
    """
    def __init__(self, should_tag: bool = none):
        self.should_tag = should_tag


class ResourceTagSpecification:
    """
    # Arguments
    volumes: TagSpecification
    snapshots: TagSpecification
    enis: TagSpecification
    amis: TagSpecification
    """
    def __init__(self, volumes: TagSpecification = none, snapshots: TagSpecification = none, enis: TagSpecification = none,
                 amis: TagSpecification = none):
        self.volumes = volumes
        self.snapshots = snapshots
        self.enis = enis
        self.amis = amis


class Tag:
    """
    # Arguments
    tag_key: str
    tag_value: str
    """
    def __init__(self, tag_key: str = none, tag_value: str = none):
        self.tag_key = tag_key
        self.tag_value = tag_value


class LaunchSpecification:
    """
    # Arguments
    instance_types: InstanceTypes
    ebs_optimized: bool
    monitoring: bool
    tenancy: str
    iam_role: IamRole
    security_group_ids: list[str]
    image_id: str
    key_pair: str
    tags: list[Tag]
    resource_tag_specification: ResourceTagSpecification
    user_data: str
    shutdown_script: str
    credit_specification: CreditSpecification
    network_interfaces: list[NetworkInterface]
    block_device_mappings: list[BlockDeviceMapping]
    """
    def __init__(self, instance_types: InstanceTypes = none, ebs_optimized: bool = none, monitoring: bool = none,
                 tenancy: str = none,
                 iam_role: IamRole = none, security_group_ids: List[str] = none, image_id: str = none,
                 key_pair: str = none, tags: List[Tag] = none,
                 resource_tag_specification: ResourceTagSpecification = none, user_data: str = none,
                 shutdown_script: str = none,
                 credit_specification: CreditSpecification = none, network_interfaces: List[NetworkInterface] = none,
                 block_device_mappings: List[BlockDeviceMapping] = none):
        self.instance_types = instance_types
        self.ebs_optimized = ebs_optimized
        self.monitoring = monitoring
        self.tenancy = tenancy
        self.iam_role = iam_role
        self.security_group_ids = security_group_ids
        self.image_id = image_id
        self.key_pair = key_pair
        self.tags = tags
        self.resource_tag_specification = resource_tag_specification
        self.user_data = user_data
        self.shutdown_script = shutdown_script
        self.block_device_mappings = block_device_mappings
        self.credit_specification = credit_specification
        self.network_interfaces = network_interfaces


class Compute:
    """
    # Arguments
    subnet_ids: list[str]
    vpc_id: str
    elastic_ip: str
    private_ip: str
    product: str
    launch_specification: LaunchSpecification
    """
    def __init__(self, subnet_ids: List[str] = none, vpc_id: str = none, elastic_ip: str = none, private_ip: str = none,
                 product: str = none,
                 launch_specification: LaunchSpecification = none):
        self.subnet_ids = subnet_ids
        self.vpc_id = vpc_id
        self.elastic_ip = elastic_ip
        self.private_ip = private_ip
        self.product = product
        self.launch_specification = launch_specification


class Route53RecordSetConfiguration:
    """
    # Arguments
    name: str
    use_public_ip: bool
    use_public_dns: bool
    """
    def __init__(self, name: str = none, use_public_ip: bool = none, use_public_dns: bool = none):
        self.name = name
        self.use_public_ip = use_public_ip
        self.use_public_dns = use_public_dns


class Route53DomainConfiguration:
    """
    # Arguments
    hosted_zone_id: str
    spotinst_account_id: str
    record_set_type: str
    record_sets: list[Route53RecordSetConfiguration]
    """
    def __init__(self, hosted_zone_id: str = none, spotinst_account_id: str = none, record_set_type: str = none,
                 record_sets: List[Route53RecordSetConfiguration] = none):
        self.hosted_zone_id = hosted_zone_id
        self.spotinst_account_id = spotinst_account_id
        self.record_set_type = record_set_type
        self.record_sets = record_sets


class Route53Configuration:
    """
    # Arguments
    domains: list[Route53DomainsConfiguration]
    """
    def __init__(self, domains: List[Route53DomainConfiguration] = none):
        self.domains = domains


class LoadBalancer:
    """
    # Arguments
    name: str
    arn: str
    type: str
    balancer_id: str
    target_set_id: str
    az_awareness: bool
    auto_weight: bool
    """
    def __init__(self, name: str = none, arn: str = none, type: str = none, balancer_id: str = none,
                 target_set_id: str = none, az_awareness: bool = none,
                 auto_weight: bool = none):
        self.name = name
        self.arn = arn
        self.type = type
        self.balancer_id = balancer_id
        self.target_set_id = target_set_id
        self.az_awareness = az_awareness
        self.auto_weight = auto_weight


class LoadBalancersConfiguration:
    """
    # Arguments
    load_balancers: list[LoadBalancer]
    """

    def __init__(self, load_balancers: List[LoadBalancer] = none):
        self.load_balancers = load_balancers


class IntegrationsConfig:
    """
    # Arguments
    route53: Route53Configuration
    load_balancers_config: LoadBalancersConfiguration
    """

    def __init__(self, route53: Route53Configuration = none, load_balancers_config: LoadBalancersConfiguration = none):
        self.route53 = route53
        self.load_balancers_config = load_balancers_config


class ManagedInstance:
    """
    # Arguments
    name: str
    description: str
    region: str
    strategy: Strategy
    persistence: Persistence
    health_check: HealthCheck
    scheduling: Scheduling
    compute: Compute
    integrations: IntegrationsConfig
    """

    def __init__(self, name: str = none, description: str = none, region: str = none,
                 strategy: Strategy = none,
                 persistence: Persistence = none, health_check: HealthCheck = none, scheduling: Scheduling = none,
                 compute: Compute = none, integrations: IntegrationsConfig = none):
        self.name = name
        self.description = description
        self.region = region
        self.strategy = strategy
        self.persistence = persistence
        self.health_check = health_check
        self.compute = compute
        self.scheduling = scheduling
        self.integrations = integrations

# region Client Requests
class ManagedInstanceCreationRequest:
    def __init__(self, managed_instance: ManagedInstance):
        self.managed_instance = managed_instance

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class DeallocationConfig:
    def __init__(self, should_delete_network_interfaces: bool = none,
                 should_delete_volumes: bool = none, should_delete_snapshots: bool = none, should_delete_images: bool = none,
                 should_terminate_instance: bool = none):
        self.should_delete_network_interfaces = should_delete_network_interfaces
        self.should_delete_volumes = should_delete_volumes
        self.should_delete_snapshots = should_delete_snapshots
        self.should_delete_images = should_delete_images
        self.should_terminate_instance = should_terminate_instance
        

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class AmiBackup:
    def __init__(self, should_delete_images: bool = none):
        self.should_delete_images = should_delete_images

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ManagedInstanceDeletionRequest:
    def __init__(self, deallocation_config: DeallocationConfig = none, ami_backup: AmiBackup = none):
        self.deallocation_config = deallocation_config
        self.ami_backup = ami_backup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ManagedInstanceUpdateRequest:
    def __init__(self, managed_instance: ManagedInstance):
        self.managed_instance = managed_instance

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)


class ManagedInstanceState:
    def __init__(self, id: str = none, state: str = none):
        self.id = id
        self.state = state  


class ManagedInstanceUpdateStatesRequest:
    def __init__(self, managed_instance_states: List[ManagedInstanceState] = none):
        self.managed_instance_states = managed_instance_states

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
                        sort_keys=True, indent=4)

class AvailabilityZones:
    def __init__(self, name: str = None, subnet_ids: List[str] = None):
        self.name = name
        self.subnet_ids = subnet_ids

class ManagedInstanceMigration:
    def __init__(self, should_keep_private_ip: bool = none, original_instance_id: str = none,
                region: str = none, should_terminate_instance: bool = none, managed_instance_name: str =none,
                product: str = none, spot_instances_types: List[str] = none, availability_zones: List[AvailabilityZones] = none):
        self.should_keep_private_ip = should_keep_private_ip
        self.original_instance_id = original_instance_id
        self.region = region
        self.should_terminate_instance = should_terminate_instance
        self.managed_instance_name = managed_instance_name
        self.product = product
        self.spot_instance_types = spot_instances_types
        self.availability_zones = availability_zones

class ManagedInstanceBulkMigrationRequest:
    def __init__(self, migrations: List[ManagedInstanceMigration] = none):
        self.migrations = migrations
    
    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
                        sort_keys=True, indent=4)

class ManagedInstanceBulkMigrationStatusRequest:
    def __init__(self, migration_ids: List[str] = none):
        self.migration_ids = migration_ids
    
    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, 
                        sort_keys=True, indent=4)
# endregion


import json

from spotinst_sdk2.client import Client

import spotinst_sdk2.models.stateful_node as azure_stateful_node

none = "d3043820717d74d9a17694c176d39733"


class StatefulNodeAzureClient(Client):
    __base_stateful_node_url = "/azure/compute/statefulNode"
    ENTITY_NAME = 'statefulNode'

    def create_stateful_node(self, node: azure_stateful_node.StatefulNode):
        """
        Create a New Stateful Node

        # Arguments
        node (StatefulNode): StatefulNode Object

        # Returns
        (Object): StatefulNode API response
        """
        request = azure_stateful_node.CreateStatefulNodeRequest(node)

        excluded_node_dict = self.exclude_missing(json.loads(request.toJSON()))

        formatted_node_dict = self.convert_json(
            excluded_node_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_stateful_node_url,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def update_stateful_node(self, node_update: azure_stateful_node.StatefulNode, node_id: str):
        """
        Update a Stateful Node

        # Arguments
        node_id (String): Stateful Node  ID
        node_update (StatefulNode): StatefulNode Object

        # Returns
        (Object): StatefulNode API response
        """
        request = azure_stateful_node.UpdateStatefulNodeRequest(node_update)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_stateful_node_url + "/" + node_id,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_stateful_node(self, node_id: str, deallocation_config: azure_stateful_node.DeallocationConfig):
        """
        Delete a Stateful Node

        # Arguments
        node_id (String): Stateful Node ID
        deallocation_config (DeallocationConfig): DeallocationConfig object

        # Returns
        (Object): StatefulNode API response
        """
        request = azure_stateful_node.DeleteStatefulNodeRequest(
            deallocation_config)

        excluded_node_delete_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_delete_dict = self.convert_json(
            excluded_node_delete_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_delete_dict)

        response = self.send_delete_with_body(
            body=body_json,
            url=self.__base_stateful_node_url + "/" + node_id,
            entity_name=self.ENTITY_NAME)

        return response

    def get_stateful_node(self, node_id: str):
        """
        Get a Stateful Node

        # Arguments
        node_id (String): Stateful Node ID

        # Returns
        (Object): Stateful Node API response
        """
        response = self.send_get(
            url=self.__base_stateful_node_url + "/" + node_id,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_stateful_nodes(self):
        """
        Get all Stateful Nodes

        # Returns
        (List): List of Stateful Nodes API response
        """
        response = self.send_get(
            url=self.__base_stateful_node_url,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def get_stateful_node_resources(self, node_id: str):
        """
        Get the node's attached resources (storage and network interfaces)

        # Arguments
        node_id (String): Stateful Node ID

        # Returns
        (Object): Stateful Node API response
        """
        response = self.send_get(
            url=self.__base_stateful_node_url + "/" + node_id + "/resources",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_stateful_node_status(self, node_id: str):
        """
        Get the status of a specific stateful node.

        # Arguments
        node_id (String): Stateful Node ID

        # Returns
        (Object): Stateful Node API response
        """
        response = self.send_get(
            url=self.__base_stateful_node_url + "/" + node_id + "/status",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_all_stateful_node_statuses(self):
        """
        Get the statuses of all Stateful Nodes in a specific account.

        # Returns
        (Object): List of Stateful Node Statuses
        """
        response = self.send_get(
            url=self.__base_stateful_node_url + "/status",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def update_stateful_node_state(self, node_id: str, state: str):
        """
        Update a Stateful Node State

        # Arguments
        node_id (String): Stateful Node  ID
        state (String): Desired state

        # Returns
        (Object): StatefulNode API response
        """
        body_json = json.dumps(dict(state=state))

        response = self.send_put(
            body=body_json,
            url=self.__base_stateful_node_url + "/" + node_id + "/state",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def get_stateful_node_from_azure_vm(self, resource_group_name: str, virtual_machine_name: str):
        """
        Get the configuration of a stateful node from a Azure VM for importing.

        # Arguments
        resource_group_name (String): Resource Group of the VM to Import
        virtual_machine_name (String): Virtual Machine to import

        # Returns
        (Object): Stateful Node API response
        """
        response = self.send_get(
            url=self.__base_stateful_node_url + "/resourceGroup/" + resource_group_name +
            "/virtualMachine/" + virtual_machine_name + "/importConfiguration",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def import_vm_to_stateful_node(self, import_vm_configuration: azure_stateful_node.ImportVmConfiguration):
        """
        Import an Azure VM and create a Stateful Node by providing a node configuration.

        # Arguments
        import_vm_configuration (StatefulNode): Configuration of VM to import

        # Returns
        (Object): StatefulNode API response
        """
        request = azure_stateful_node.ImportVmToStatefulNodeRequest(
            import_vm_configuration=import_vm_configuration)

        excluded_node_dict = self.exclude_missing(json.loads(request.toJSON()))

        formatted_node_dict = self.convert_json(
            excluded_node_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_dict)

        response = self.send_post(
            body=body_json,
            url=self.__base_stateful_node_url + "/import",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_stateful_node_import_status(self, import_id):
        """
        Get the import process status of a Stateful Node.

        # Arguments
        import_id (String): Import ID

        # Returns
        (Object) : Stateful Node API response
        """
        response = self.send_get(
            url=self.__base_stateful_node_url + "/import/" + import_id + "/status",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def attach_data_disk_to_stateful_node(self, node_id: str,
                                          data_disk_configuration: azure_stateful_node.AttachDataDiskConfiguration):
        """
        Create a new data disk and attche it to the Stateful Node.

        # Arguments
        node_id (String): Stateful Node  ID
        data_disk_configuration (AttachDataDiskConfiguration): Configuration of Data Disk

        # Returns
        (Object): StatefulNode API response
        """
        request = azure_stateful_node.AttachDataDiskToStatefulNodeRequest(
            data_disk_configuration)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_stateful_node_url + "/" + node_id + "/dataDisk/attach",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]

    def detach_data_disk_from_stateful_node(self, node_id: str,
                                            data_disk_configuration: azure_stateful_node.DetachDataDiskConfiguration):
        """
        Detach a data disk from a Stateful Node.

        # Arguments
        node_id (String): Stateful Node  ID
        data_disk_configuration (DetachDataDiskConfiguration): Configuration of Data Disk

        # Returns
        (Object): StatefulNode API response
        """
        request = azure_stateful_node.DetachDataDiskFromStatefulNodeRequest(
            data_disk_configuration)

        excluded_node_update_dict = self.exclude_missing(
            json.loads(request.toJSON()))

        formatted_node_update_dict = self.convert_json(
            excluded_node_update_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_node_update_dict)

        response = self.send_put(
            body=body_json,
            url=self.__base_stateful_node_url + "/" + node_id + "/dataDisk/detach",
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def get_stateful_node_logs(self, node_id, from_date, to_date, severity=None, resource_id=None, limit=None):
        """
        Get the logs of a Stateful Node according to severity and time period filter parameters.

        # Arguments
        node_id (String): Stateful ID
        to_date (String): On or Before this date 
        from_date (String): On or After this date
        severity(String) (Optional): Log level severity
        resource_id(String) (Optional): Filter log extracted entires related to a specific resource id
        limit(String) (Optional): Maximum number of lines to extract in a response

        # Returns
        (Object): Stateful Node API response
        """
        query_params = dict(toDate=to_date, fromDate=from_date, severity=severity,
                            resourceId=resource_id, limit=limit)

        response = self.send_get(
            url=self.__base_stateful_node_url + "/" + node_id + "/log",
            query_params=query_params,
            entity_name=self.ENTITY_NAME)

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]



import json

none = "d3043820717d74d9a17694c176d39733"


# region Elastigroup
class Elastigroup:
    """
    # Arguments
    name: str
    description: str
    capacity: Capacity
    strategy: Strategy
    scaling: Scaling
    third_parties_integration: ThirdPartiesIntegration
    compute: Compute
    """
    def __init__(
            self,
            name=none,
            description=none,
            capacity=none,
            strategy=none,
            scaling=none,
            third_parties_integration=none,
            compute=none):

        self.name = name
        self.description = description
        self.capacity = capacity
        self.strategy = strategy
        self.scaling = scaling
        self.third_parties_integration = third_parties_integration
        self.compute = compute

# endregion

# region Capacity
class Capacity:
    """
    # Arguments
    minimum: int
    maximum: int
    target: int
    """
    def __init__(self, minimum=none, maximum=none, target=none):

        self.minimum = minimum
        self.maximum = maximum
        self.target = target
# endregion


# region Strategy
class Strategy:
    """
    # Arguments
    preemptible_percentage: int
    on_demand_count: int
    draining_timeout: int
    fallback_to_od: bool
    """
    def __init__(
            self,
            preemptible_percentage=none,
            on_demand_count=none,
            draining_timeout=none,
            fallback_to_od=none):

            self.preemptible_percentage = preemptible_percentage
            self.on_demand_count = on_demand_count
            self.draining_timeout = draining_timeout
            self.fallback_to_od = fallback_to_od
# endregion


# region Scaling
class Scaling:
    """
    # Arguments
    up:  list[ScalingPolicy]
    down: list[ScalingPolicy]
    """
    def __init__(self, up=none, down=none):

        self.up = up
        self.down = down

class ScalingPolicy:
    """
    # Arguments
    source: str
    policy_name: str
    namespace: str
    metric_name: str
    dimensions: list[ScalingPolicyDimension]
    statistic: str
    unit: str
    threshold: float
    period: int
    evaluation_periods: int
    cooldown: int
    action: ScalingPolicyAction
    operator: str
    """
    def __init__(
            self,
            source=none,
            policy_name=none,
            namespace=none,
            metric_name=none,
            dimensions=none,
            statistic=none,
            unit=none,
            threshold=none,
            period=none,
            evaluation_periods=none,
            cooldown=none,
            action=none,
            operator=none):

        self.source = source
        self.policy_name = policy_name
        self.namespace = namespace
        self.metric_name = metric_name
        self.dimensions = dimensions
        self.statistic = statistic
        self.unit = unit
        self.threshold = threshold
        self.period = period
        self.evaluation_periods = evaluation_periods
        self.cooldown = cooldown
        self.action = action
        self.operator = operator

class ScalingPolicyDimension:
    """
    # Arguments
    name: str
    value: str
    """
    def __init__(self, name=none, value=none):

        self.name = name
        self.value = value

class ScalingPolicyAction:
    """
    # Arguments
    scaling_type: str
    adjustment: int
    """
    def __init__(self, scaling_type=none, adjustment=none):

        self.type = scaling_type
        self.adjustment = adjustment
# endregion


# region ThirdPartiesIntegration
class ThirdPartiesIntegration:
    """
    # Arguments
    docker_swarm: DockerSwarmConfiguration
    gke : GKE
    """
    def __init__(
            self,
            docker_swarm=none,
            gke=none):

        self.docker_swarm = docker_swarm
        self.gke = gke

class DockerSwarmConfiguration:
    """
    # Arguments
    master_host: str
    master_port: int
    """
    def __init__(self, master_host=none, master_port=none):

        self.master_host = master_host
        self.master_port = master_port

class GKE:
    """
    # Arguments
    auto_update: bool
    auto_scale: AutoScale
    """
    def __init__(
            self,
            auto_update=none,
            auto_scale=none):

        self.auto_update = auto_update
        self.auto_scale = auto_scale

class AutoScale:
    """
    # Arguments
    is_enabled: bool
    is_auto_config: bool
    cooldown: int
    headroom: Headroom
    labels: list[Label]
    down: Down
    """
    def __init__(
            self,
            is_enabled=none,
            is_auto_config=none,
            cooldown=none,
            headroom=none,
            labels=none,
            down=none):

        self.is_enabled = is_enabled
        self.is_auto_config = is_auto_config
        self.cooldown = cooldown
        self.headroom = headroom
        self.labels = labels
        self.down = down

class Headroom:
    """
    # Arguments
    cpu_per_unit: int
    memory_per_unit: int
    num_of_units: int
    """
    def __init__(
            self,
            cpu_per_unit=none,
            memory_per_unit=none,
            num_of_units=none):

        self.cpu_per_unit = cpu_per_unit
        self.memory_per_unit = memory_per_unit
        self.num_of_units = num_of_units

class Down:
    """
    # Arguments
    evaluation_periods: int
    """
    def __init__(self, evaluation_periods=none):
        self.evaluation_periods = evaluation_periods


# endregion


# region Compute
class Compute:
    """
    # Arguments
    launch_specification: LaunchSpecification
    instance_types: InstanceTypes
    gpu: Gpu
    health: Health
    availability_zones: list[str]
    subnets: list[Subnet]
    """
    def __init__(
            self,
            launch_specification=none,
            instance_types=none,
            gpu=none,
            health=none,
            availability_zones=none,
            subnets=none):

        self.launch_specification = launch_specification
        self.instance_types = instance_types
        self.gpu = gpu
        self.health = health
        self.availability_zones = availability_zones
        self.subnets = subnets

class LaunchSpecification:
    """
    # Arguments
    labels: list[Label]
    metadata: list[MetaData]
    tags: List[str]
    backend_service_config: BackendServiceConfig
    startup_script: str
    disks: list[Disk]
    network_interfaces: list[NetworkInterface]
    service_account: str
    ip_forwarding: bool
    """
    def __init__(
            self,
            labels=none,
            metadata=none,
            tags=none,
            backend_service_config=none,
            startup_script=none,
            disks=none,
            network_interfaces=none,
            service_account=none,
            ip_forwarding=none):

        self.labels = labels
        self.metadata = metadata 
        self.tags = tags 
        self.backend_service_config = backend_service_config
        self.startup_script = startup_script
        self.disks = disks
        self.network_interfaces = network_interfaces
        self.service_account = service_account
        self.ip_forwarding = ip_forwarding

class Label:
    """
    # Arguments
    key: str
    value: str
    """
    def __init__(self, key=none, value=none):

        self.key = key
        self.value = value

class Metadata:
    """
    # Arguments
    key: str
    value: str
    """
    def __init__(self, key=none, value=none):

        self.key = key
        self.value = value

class BackendServiceConfig:
    """
    # Arguments
    backend_services: [BackendServices]
    """
    def __init__(self, backend_services=none):
        self.backend_services = backend_services

class BackendServices:
    """
    # Arguments
    backend_service_name: str
    location_type: str
    scheme: str
    named_ports: NamedPorts
    """
    def __init__(
            self,
            backend_service_name=none,
            location_type=none,
            scheme=none,
            named_ports=none):

        self.backend_service_name = backend_service_name
        self.location_type = location_type
        self.scheme = scheme
        self.named_ports = named_ports

class NamedPorts:
    """
    # Arguments
    name: str
    ports: list[int]
    """
    def __init__(
            self,
            name=none,
            ports=none):

        self.name = name
        self.ports = ports

class Disk:
    """
    # Arguments
    auto_delete: bool
    boot: bool
    device_name: str
    initialize_params: InitializeParams
    interface: str
    mode: str
    source: str
    disk_type: str
    """
    def __init__(
            self,
            auto_delete=none,
            boot=none,
            device_name=none,
            initialize_params=none,
            interface=none,
            mode=none,
            source=none,
            disk_type=none):

        self.auto_delete = auto_delete
        self.boot = boot
        self.device_name = device_name
        self.initialize_params = initialize_params
        self.interface = interface
        self.mode = mode
        self.source = source
        self.type = disk_type

class InitializeParams:
    """
    # Arguments
    disk_size_gb: int
    disk_type: str
    source_image: str
    """

    def __init__(
            self,
            disk_size_gb=none,
            disk_type=none,
            source_image=none):

        self.disk_size_gb = disk_size_gb
        self.disk_type = disk_type
        self.source_image = source_image

class NetworkInterface: 
    """
    # Arguments
    network: str
    access_configs: list[AccessConfig]
    alias_ip_ranges: list[AliasIpRange]
    """
    def __init__(
            self,
            network=none,
            access_configs=none,
            alias_ip_ranges=none):

        self.network = network
        self.access_configs = access_configs
        self.alias_ip_ranges = alias_ip_ranges

class AccessConfig:
    """
    # Arguments
    name: str
    access_type: str
    """
    def __init__(
            self,
            name=none,
            access_type=none):

        self.name = name
        self.type = access_type

class AliasIpRange:
    """
    # Arguments
    ip_cidr_range: str
    subnetwork_range_name: str
    """
    def __init__(
            self,
            ip_cidr_range=none,
            subnetwork_range_name=none):

        self.ip_cidr_range = ip_cidr_range
        self.subnetwork_range_name = subnetwork_range_name

class InstanceTypes:
    """
    # Arguments
    ondemand: str
    preemptible: list[str]
    custom: list[CustomInstanceTypes]
    """
    def __init__(
            self,
            ondemand=none,
            preemptible=none,
            custom=none):

        self.ondemand = ondemand
        self.preemptible = preemptible
        self.custom = custom

class CustomInstanceTypes:
    """
    # Arguments
    v_cpu: int
    memory_gib: int
    """
    def __init__(
            self,
            v_cpu=none,
            memory_gib=none):

        self.v_cPU = v_cpu
        self.memory_giB = memory_gib

class Gpu:
    """
    # Arguments
    gpu_type: str
    count: int
    """
    def __init__(
            self,
            gpu_type=none,
            count=none):
        self.type = gpu_type
        self.count = count

class Health:
    """
    # Arguments
    grace_period: int
    """
    def __init__(
            self,
            grace_period=none):
        self.grace_period = grace_period

class Subnet:
    """
    # Arguments
    region: str
    subnet_names: list[str]
    """
    def __init__(
            self,
            region=none,
            subnet_names=none):

        self.region = region
        self.subnet_names = subnet_names

class ElastigroupCreationRequest:
    def __init__(self, elastigroup):
        self.group = elastigroup

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion

# Deployment
class RollGroup:
    """
    # Arguments
    batch_size_percentage: int
    grace_period: int
    """
    def __init__(
            self,
            batch_size_percentage=none,
            grace_period=none):

        self.batch_size_percentage = batch_size_percentage
        self.grace_period = grace_period

class ElastigroupRollRequest:
    def __init__(self, group_roll):
        self.batch_size_percentage = group_roll.batch_size_percentage
        self.grace_period = group_roll.grace_period

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__,
                          sort_keys=True, indent=4)
# endregion


# region Detach Instance
class DetachConfiguration:
    """
    # Arguments
    instances_to_detach: list[str]
    should_terminate_instances: bool
    draining_timeout: int
    should_decrement_target_capacity: bool
    """
    def __init__(
            self,
            instances_to_detach=none,
            should_terminate_instances=none,
            draining_timeout=none,
            should_decrement_target_capacity=none):
        self.instances_to_detach = instances_to_detach
        self.should_terminate_instances = should_terminate_instances
        self.draining_timeout = draining_timeout
        self.should_decrement_target_capacity = should_decrement_target_capacity

class ElastigroupDetachInstancesRequest:
    def __init__(self, detach_configuration):
        self.should_decrement_target_capacity = detach_configuration.should_decrement_target_capacity
        self.draining_timeout = detach_configuration.draining_timeout
        self.instances_to_detach = detach_configuration.instances_to_detach
        self.should_terminate_instances = detach_configuration.should_terminate_instances

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)
# endregion





import json

from spotinst_sdk2.client import Client
import spotinst_sdk2.models.admin.user_mapping as spotinst_user_mapping

class AdminClient(Client):
    __base_setup_url = "/setup"

    # region Organization and Account
    def create_organization(self, org_name):
        """
        Create an organization 
        
        # Arguments
        org_name (String): Orgnanization name
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url= self.__base_setup_url +
            "/organization",
            body=json.dumps(dict(organization=dict(name=org_name))),
            entity_name="organization"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def delete_organization(self, org_id):
        """
        delete organization 
        
        # Arguments
        org_id (String): Organization Id
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url= self.__base_setup_url + 
            "/organization/" + str(org_id),
            entity_name="organization"
        )

        return response

    def create_aws_external_id(self):
        """
        Important note: This is deprecated, please use setup_aws client instead(SetupAWSClient#create_external_id)

        Create aws account external id.
        You should use the external id when creating your AWS role for your spot account

        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url= self.__base_setup_url + "/credentials/aws/externalId",
            entity_name="credentials"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def set_cloud_credentials(self, iam_role, external_id=None):
        """
        Important note: This is deprecated, please use setup_aws client instead(SetupAWSClient#set_credentials)

        set cloud credentials 
        Please create external id using spot api (see #AdminClient.create_aws_external_id)
        and use it when creating the AWS role
        
        # Arguments
        iam_role (String): IAM Role
        external_id (String) (Optional): External ID
        
        # Returns
        (Object): Spotinst API response 
        """
        credentials = {"iamRole": iam_role}

        if external_id is not None:
            credentials['externalId'] = external_id

        response = self.send_post(
            url= self.__base_setup_url +
            "/credentials/aws",
            body=json.dumps(dict(credentials=credentials)),
            entity_name="credentials"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def create_account(self, account_name):
        """
        create an account 
        
        # Arguments
        account_name (String): Account Name
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url= self.__base_setup_url +
            "/account",
            body=json.dumps(dict(account=dict(name=account_name))),
            entity_name="account"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def get_accounts(self):
        """
        get accounts in organization
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_get(
            url= self.__base_setup_url +
            "/account",
            entity_name="account"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def delete_account(self, account_name):
        """
        delete account
        
        # Arguments
        account_name (String): Account Name
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete(
            url= self.__base_setup_url +
            "/account/" + account_name,
            entity_name="account"
        )

        return response

    def create_user(self, first_name, last_name, email, password, role):
        """
        Create user
        
        # Arguments
        first_name (String): Users first name
        last_name (String): User last name
        email (String): Eser email
        password (String): User email
        role (String): User role
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url= self.__base_setup_url +
            "/user",
            body=json.dumps(dict(
                firstName=first_name,
                lastName=last_name,
                email=email,
                password=password,
                role=role)),
            entity_name="user"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"][0]

    def add_exsisting_user(self, user_email, role):
        """
        Add exsisting user
        
        # Arguments
        user_email (String): User email
        role (String): User role
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_post(
            url= self.__base_setup_url +
            "/account/" + self.account_id +
            "/user",
            body=json.dumps(dict(userEmail=user_email, role=role)),
            entity_name="user"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def update_user_role(self, user_email, role):
        """
        Update exsisting user

        # Arguments
        user_email (String): User email
        role (String): User role
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_put(
            url= self.__base_setup_url +
            "/account/" + self.account_id +
            "/user",
            body=json.dumps(dict(userEmail=user_email, role=role)),
            entity_name="user"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    def detach_user(self, user_email):
        """
        Delete exsisting user
        
        # Arguments
        user_email (String): User email
        
        # Returns
        (Object): Spotinst API response 
        """
        response = self.send_delete_with_body(
            url= self.__base_setup_url +
            "/account/" + self.account_id +
            "/user",
            body=json.dumps(dict(userEmail=user_email)),
            entity_name="user"
        )

        return response

    def get_user(self, user_email):
        """
        Get user
        
        # Arguments
        user_email (String): User email
        
        # Returns
        (Object): Spotinst API response 
        """
        query_params= dict(userEmail=user_email)
        response = self.send_get(
            url= self.__base_setup_url + "/accountUserMapping",
            query_params=query_params,
            entity_name="user"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["items"]

    def assign_user_to_account(self, mappings):
        """
        Assign user to account
        
        # Arguments
        mappings (List): List of UserMapping Objects
        
        # Returns
        (Object): Spotinst API response 
        """
        mappings = spotinst_user_mapping.UserMappingRequest(mappings)

        excluded_group_dict = self.exclude_missing(json.loads(mappings.toJSON()))

        formatted_group_dict = self.convert_json(
            excluded_group_dict, self.underscore_to_camel)

        body_json = json.dumps(formatted_group_dict)

        response = self.send_post(
            url= self.__base_setup_url + "/accountUserMapping",
            body= body_json,
            entity_name="user"
        )

        formatted_response = self.convert_json(
            response, self.camel_to_underscore)

        return formatted_response["response"]["status"]

    # endregion